package com.psol.xcommerce;

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Simple console to create database and enter data in
 * the SQL database.
 *
 * @version Dec 23, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class XMLServerConsole
   extends HttpServlet
{
   /**
    * handles GET request
    * @param request HTTP request
    * @param response hold the response
    * @exception SerlvetException error handling the request
    * @exception IOException error writing the reply
    */
   protected void doGet(HttpServletRequest request,
                        HttpServletResponse response)
      throws ServletException, IOException
   {
      doProcess(request,response);
   }

   /**
    * handles GET request
    * @param request HTTP request
    * @param response hold the response
    * @exception SerlvetException error handling the request
    * @exception IOException error writing the reply
    */
   protected void doPost(HttpServletRequest request,
                         HttpServletResponse response)
      throws ServletException, IOException
   {
      doProcess(request,response);
   }

   /**
    * GET and POST requests are forwarded here
    * @param request HTTP request
    * @param response hold the response
    * @exception SerlvetException error handling the request
    * @exception IOException error writing the reply
    */
   protected void doProcess(HttpServletRequest request,
                            HttpServletResponse response)
      throws ServletException, IOException
   {
      String sqlDriver = getInitParameter("sql.driver"),
             sqlURL = getInitParameter("sql.url"),
             sqlUser = getInitParameter("sql.user"),
             sqlPassword = getInitParameter("sql.password");
      try
      {
         Class.forName(sqlDriver);
         Connection connection =
            DriverManager.getConnection(sqlURL,
                                        sqlUser,
                                        sqlPassword);
         try
         {
            String action = request.getParameter("action");
            if(null != action)
            {
               if(action.equalsIgnoreCase("create"))
                  doUpdates(request,connection,createStatements);
               else if(action.equalsIgnoreCase("drop"))
                  doUpdates(request,connection,dropStatements);
               else if(action.equalsIgnoreCase("delete"))
                  doDelete(request,connection);
               else if(action.equalsIgnoreCase("insert"))
                  doInsert(request,connection);
            }
            doPage(request,response,connection);
         }
         finally
         {
            connection.close();
         }
      }
      catch(Exception e)
      {
         throw new ServletException(e);
      }
   }

   /**
    * drop and create statements to delete/create the database
    */
   private static final String[] dropStatements =
   {
      "drop table products",
      "drop table orders",
   };
   private static final String[] createStatements =
   {
      "create table products (id integer not null constraint idconstraint primary key," +
      "name varchar(50),manufacturer varchar(50)," +
      "img varchar(30),warranty varchar(20)," +
      "description varchar(150),price real)",
      "create table orders (name varchar(50)," +
      "street varchar(100),region varchar(50)," +
      "postal_code varchar(15),locality varchar(50)," +
      "country varchar(25),email varchar(50)," +
      "productid integer,productname varchar(50)," +
      "productprice real,productquantity integer)"
   };

   /**
    * execute a number of updates on the database
    * (typically to create schema)
    * @param request HTTP request
    * @param connection database connection
    * @param statements statements to execute
    * @throw SQLException one statement throw an exception
    */
   protected void doUpdates(HttpServletRequest request,
                            Connection connection,
                            String[] statements)
      throws SQLException
   {
      Statement stmt = connection.createStatement();
      SQLException e = null;
      try
      {
         for(int i = 0;i < statements.length;i++)
            try
               { stmt.executeUpdate(statements[i]); }
            catch(SQLException x)
               { e = e != null ? e : x; }
         if(null != e)
         {
            throw e;
         }
      }
      finally
      {
         stmt.close();
      }
   }

   /**
    * delete one product from the database
    * @param request HTTP request
    * @param connection database connection
    * @return the form to display (the result screen if you like)
    */
   protected void doDelete(HttpServletRequest request,
                           Connection connection)
      throws SQLException
   {
      PreparedStatement stmt =
         connection.prepareStatement(
            "delete from products where id = ?");
      try
      {
         String id = request.getParameter("id");
         stmt.setInt(1,Integer.parseInt(id));
         stmt.executeUpdate();
      }
      finally
      {
         stmt.close();
      }
   }

   /**
    * create a new product in the database
    * @param request HTTP request
    * @param connection database connection
    * @return the form to display (the result screen if you like)
    */
   protected void doInsert(HttpServletRequest request,
                           Connection connection)
      throws SQLException, Exception
   {
      String id = request.getParameter("id"),
             name = request.getParameter("name"),
             manufacturer = request.getParameter("manufacturer"),
             image = request.getParameter("image"),
             warranty = request.getParameter("warranty"),
             description = request.getParameter("description"),
             price = request.getParameter("price");
      PreparedStatement stmt =
         connection.prepareStatement(
            "insert into products (id,name,manufacturer,img," +
            "warranty,description,price) values(?,?,?,?,?,?,?)");
      try
      {
         stmt.setString(1,id);
         stmt.setString(2,name);
         stmt.setString(3,manufacturer);
         stmt.setString(4,image);
         stmt.setString(5,warranty);
         stmt.setString(6,description);
         stmt.setString(7,price);
         stmt.executeUpdate();
      }
      finally
      {
         stmt.close();
      }
   }

   /**
    * check whether the schema has been created
    * @return true if the schema is complete, false otherwise
    */
   protected boolean isSchemaCreated(Connection connection)
      throws SQLException
   {
      // ask the name of all the tables in the database
      // check if one of them is "products"
      DatabaseMetaData meta = connection.getMetaData();
      ResultSet rs =
         meta.getTables(null,null,null,new String[] { "TABLE" });
      int found = 0;
      while(rs.next())
      {
         String tableName = rs.getString("TABLE_NAME");
         if(tableName.equalsIgnoreCase("products")
            || tableName.equalsIgnoreCase("orders"))
            found++;
      }
      rs.close();
      return 2 == found;
   }

   /**
    * display the page, etc.
    * @param request HTTP request
    * @param connection database connection
    */
   protected void doPage(HttpServletRequest request,
                         HttpServletResponse response,
                         Connection connection)
      throws SQLException, IOException
   {
      Writer writer = response.getWriter();
      writer.write("<HTML><HEAD><TITLE>XML Server Console" +
         "</TITLE></HEAD><BODY>");
      Statement stmt = connection.createStatement();
      try
      {
      
         if(isSchemaCreated(connection))
         {
            writer.write("<P><FORM ACTION=\"");
            writer.write(request.getServletPath());
            writer.write("\" METHOD=\"POST\"><TABLE>");
            writer.write("<TR><TD>Identifier:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"id\"></TD></TR>");
            writer.write("<TR><TD>Name:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"name\"></TD></TR>");
            writer.write("<TR><TD>Manufacturer:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"manufacturer\"></TD></TR>");
            writer.write("<TR><TD>Image:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"image\"></TD></TR>");
            writer.write("<TR><TD>Warranty:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"warranty\"></TD></TR>");
            writer.write("<TR><TD>Description:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"description\"></TD></TR>");
            writer.write("<TR><TD>Price:</TD>");
            writer.write("<TD><INPUT TYPE=\"TEXT\"");
            writer.write(" NAME=\"price\"></TD></TR>");
            writer.write("</TABLE><INPUT TYPE=\"SUBMIT\"");
            writer.write(" VALUE=\"Create\">");
            writer.write("<INPUT TYPE=\"HIDDEN\"");
            writer.write(" NAME=\"action\" VALUE=\"insert\">");
            writer.write("</FORM><P>");
            ResultSet rs =
               stmt.executeQuery("select id, name from products");
            writer.write("<TABLE>");
            while(rs.next())
            {
               writer.write("<TR><TD>");
               writer.write(rs.getString(2));
               writer.write("</TD><TD><FORM ACTION=\"");
               writer.write(request.getServletPath());
               writer.write("\" METHOD=\"POST\">");
               writer.write(" <INPUT TYPE=\"SUBMIT\"");
               writer.write(" VALUE=\"Delete\">");
               writer.write("<INPUT TYPE=\"HIDDEN\"");
               writer.write(" NAME=\"action\" VALUE=\"delete\">");
               writer.write("<INPUT TYPE=\"HIDDEN\"");
               writer.write(" NAME=\"id\" VALUE=\"");
               writer.write(rs.getString(1));
               writer.write("\">");
               writer.write("</FORM></TD></TR>");
            }
            writer.write("</TABLE>");
            rs = stmt.executeQuery("select name, " +
                    "productname from orders");
            writer.write("<TABLE>");
            while(rs.next())
            {
               writer.write("<TR><TD>");
               writer.write(rs.getString(1));
               writer.write("</TD><TD>");
               writer.write(rs.getString(2));
               writer.write("</TD></TR>");
            }
            writer.write("</TABLE>");
         }
         writer.write("<P><FORM ACTION=\"");
         writer.write(request.getServletPath());
         writer.write("\" METHOD=\"POST\">");
         writer.write("<INPUT TYPE=\"SUBMIT\"");
         writer.write(" VALUE=\"Drop tables\">");
         writer.write("<INPUT TYPE=\"HIDDEN\"");
         writer.write(" NAME=\"action\" VALUE=\"drop\">");
         writer.write("</FORM>");
         writer.write("<FORM ACTION=\"");
         writer.write(request.getServletPath());
         writer.write("\" METHOD=\"POST\">");
         writer.write("<INPUT TYPE=\"SUBMIT\"");
         writer.write(" VALUE=\"Create tables\">");
         writer.write("<INPUT TYPE=\"HIDDEN\"");
         writer.write(" NAME=\"action\" VALUE=\"create\">");
         writer.write("</FORM>");
      }
      finally
      {
         stmt.close();
      }
      writer.write("</BODY></HTML>");
      writer.flush();
   }
}